#include <Menu.h>

void Menu::updateMenu()
{
  QFile *configFile = new QFile(filename);
  if (! configFile->open(QIODevice::ReadOnly | QFile::Text))
  {
    qDebug() << "Error: Couldn't open the configuration file.";
    qDebug() << configFile->errorString();
    return;
  }
  QTextStream in(configFile);
  QString line;
  clear();
  quitAct = new QAction(QIcon(":/images/exit.svg"), tr("Quit"), this);
  connect(quitAct, SIGNAL(triggered(bool)), app, SLOT(quit()));
  int lineno = 1;
  ExecAction *act = NULL;
  QMenu *currentSubmenu = this;

  for (line = in.readLine().trimmed(); ! line.isNull(); ++lineno)
  {
    nextLine(line, lineno, act, currentSubmenu);
    line = in.readLine().trimmed();
  }
  addSeparator();
  addAction(quitAct);

  configFile->close();
}

QFile *Menu::nextLine(QString line, int &lineno, ExecAction *&act, QMenu *&currentSubmenu)
{
   /* if the next entry is dynamic: return a file pointer to it
   /  else return NULL
  */

  QString tmp;

  if (line.startsWith("#") || line.isEmpty()) {}
  else
  if (line.compare("[Separator]", Qt::CaseInsensitive) == 0)
  {
    currentSubmenu->addSeparator();
  }
  else
  if (line.startsWith("[Submenu ", Qt::CaseInsensitive) && line.endsWith("]"))
  {
    tmp = line;
    tmp.remove(0, 9).chop(1);
    qDebug() << "adding submenu" << tmp;
    currentSubmenu = new QMenu(tmp, currentSubmenu);
    ((QMenu *) currentSubmenu->parent())->addMenu(currentSubmenu);
  }
  else
  if (line.startsWith("[Dynamic ", Qt::CaseInsensitive) && line.endsWith("]"))
  {
    tmp = line;
    tmp.remove(0, 9).chop(1);
    qDebug() << "adding dynamic entry" << tmp;

    FILE *fp=readDynamicEntry(tmp);
    QFile *dynamic = new QFile(this);
    dynamic->open(fp, QIODevice::ReadOnly);
    if (dynamic->isOpen())
    {
      QTextStream inD(dynamic);
      QString lineD;

      for (lineD = inD.readLine().trimmed(); ! lineD.isNull(); ++lineno)
      {
        nextLine(lineD, lineno, act, currentSubmenu);
        lineD = inD.readLine().trimmed();
      }
      pclose(fp);
    }
    else
      qDebug() << "error: could not open.";
  }
  else
  if (line.compare("[/Submenu]", Qt::CaseInsensitive) == 0)
  {
    if (currentSubmenu->parent() != 0)
      currentSubmenu = (QMenu *) currentSubmenu->parent();
  }
  else
  if (line.startsWith("[") && line.endsWith("]"))
  {
    tmp = line;
    tmp.remove(0, 1).chop(1);
    qDebug() << "adding" << tmp;
    act = new ExecAction(tmp, this);
    currentSubmenu->addAction(act);
  }
  else
  if (line.startsWith("Exec", Qt::CaseInsensitive))
  {
    if (act != NULL)
    {
      act->setCommand(line.section('=', 1).trimmed().toAscii());
      connect(act, SIGNAL(triggered(bool)), act, SLOT(runCommand()));
    }
  }
  else
  if (line.startsWith("Icon", Qt::CaseInsensitive))
  {
    if (act != NULL)
    {
      act->setIcon(QIcon(line.section('=', 1).trimmed()));
    }
  }
  else
  if (line.compare("RunAsRoot", Qt::CaseInsensitive) == 0)
  {
    if (act != NULL)
    {
      qDebug() << "run this command as root.";
      act->setRunAsRootKDE();
    }
  }
  else
  if (line.startsWith("RunAsRoot", Qt::CaseInsensitive))
  {
    if (act != NULL && line.section('=', 1).trimmed() == "KDE")
    {
      qDebug() << "run this command as root (KDE).";
      act->setRunAsRootKDE();
    }
    else
    if (act != NULL && line.section('=', 1).trimmed() == "GTK")
    {
      qDebug() << "run this command as root (GTK).";
      act->setRunAsRootGTK();
    }
  }
  else
  {
    qDebug() << "Syntax error:" << line;
  }

  return NULL;
}

FILE *Menu::readDynamicEntry(QString cmd)
{
  return popen(cmd.toAscii().constData(), "r");
}
